package java.util.stream;

import java.util.Collections;
import java.util.EnumSet;
import java.util.Objects;
import java.util.Set;
import java.util.function.Function;
import java.util.function.BinaryOperator;
import java.util.function.BiConsumer;
import java.util.function.Supplier;

public interface Collector<T, A, R>
{
    public abstract Supplier<A> supplier();
    
    public abstract BiConsumer<A, T> accumulator();
    
    public abstract BinaryOperator<A> combiner();
    
    public abstract Function<A, R> finisher();
    
    public abstract Set<Collector.Characteristics> characteristics();
    
    public static <T, R> Collector<T, R, R> of(Supplier<R> obj, BiConsumer<R, T> obj2, BinaryOperator<R> obj3, Collector.Characteristics... array) {
        Objects.requireNonNull(obj);
        Objects.requireNonNull(obj2);
        Objects.requireNonNull(obj3);
        Objects.requireNonNull(array);
        return new Collectors.CollectorImpl(obj, obj2, obj3, (array.length == 0) ? Collectors.CH_ID : Collections.unmodifiableSet(EnumSet.of(Collector.Characteristics.IDENTITY_FINISH, array)));
    }
    
    public static <T, A, R> Collector<T, A, R> of(Supplier<A> obj, BiConsumer<A, T> obj2, BinaryOperator<A> obj3, Function<A, R> obj4, Collector.Characteristics... array) {
        Objects.requireNonNull(obj);
        Objects.requireNonNull(obj2);
        Objects.requireNonNull(obj3);
        Objects.requireNonNull(obj4);
        Objects.requireNonNull(array);
        Set<Object> set = Collectors.CH_NOID;
        if (array.length > 0) {
            EnumSet<Collector.Characteristics> none = EnumSet.noneOf(Collector.Characteristics.class);
            Collections.addAll(none, array);
            set = Collections.unmodifiableSet(none);
        }
        return new Collectors.CollectorImpl(obj, obj2, obj3, obj4, set);
    }
    
    public enum Characteristics
    {
        CONCURRENT, 
        UNORDERED, 
        IDENTITY_FINISH;
    }
}
